{
David Lipschitz, Orbital Decisions (Pty) Ltd
E-mail david@orbital.co.za
URL http://www.orbital.co.za

Use at your own risk!
}
unit ODPopCal;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, ODCalend, DsgnIntf;

type
  TODCalendarForm = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    procedure FormShow(Sender: TObject);
  private
    FCalendar: TODCalendar;
    FOnStartChange, FOnFinishChange: TNotifyEvent;
    procedure StartChange(Sender: TObject);
    procedure FinishChange(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    property Calendar: TODCalendar read FCalendar write FCalendar;
    property OnStartChange: TNotifyEvent read FOnStartChange write FOnStartChange;
    property OnFinishChange: TNotifyEvent read FOnFinishChange write FOnFinishChange;
  end;

  TODCalendarDialog = class(TComponent)
  private
    FForm: TODCalendarForm;
    FHeaders: TODCalendarHeaders;
    FCaption, FDateFormat: string;
    FDialogHeight, FDialogWidth: Integer;
    FStartDate, FFinishDate: TDateTime;
    FDisplayYear, FDisplayMonth{, FDisplayWeek}: Integer;
    FDayColor, FTodayColor, FRangeColor, FWeekColor: TColor;
    FTitleFont, FStartFont, FFinishFont, FDateFont: TFont;
    FAutoPage, FSingleDate, FShowStatus, FStartOnMonday: Boolean;
    FBevelEdge: TPanelBevel;
    FOnStartChange, FOnFinishChange: TNotifyEvent;
    FOnStartClick, FOnFinishClick: TODDateEvent;
    FOnSelectYear: TODSelectYearEvent;
    FOnSetupDay: TODSetupDayEvent;
    function GetDisplayDate: TDateTime;
    procedure SetDisplayDate(Value: TDateTime);
    procedure SetDisplayYear(Value: Integer);
    procedure SetDisplayMonth(Value: Integer);
//  procedure SetDisplayWeek(Value: Integer);
    procedure SetTitleFont(Value: TFont);
    procedure SetStartFont(Value: TFont);
    procedure SetFinishFont(Value: TFont);
    procedure SetDateFont(Value: TFont);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: Boolean;
    property StartDate: TDateTime read FStartDate write FStartDate;
    property FinishDate: TDateTime read FFinishDate write FFinishDate;
    property DisplayDate: TDateTime read GetDisplayDate write SetDisplayDate;
  published
    property Caption: string read FCaption write FCaption;
    property DialogHeight: Integer read FDialogHeight write FDialogHeight default 250;
    property DialogWidth: Integer read FDialogWidth write FDialogWidth default 240;
    property DisplayYear: Integer read FDisplayYear write SetDisplayYear;
    property DisplayMonth: Integer read FDisplayMonth write SetDisplayMonth;
//  property DisplayWeek: Integer read FDisplayWeek write SetDisplayWeek;
    property DayColor: TColor read FDayColor write FDayColor default clWindow;
    property TodayColor: TColor read FTodayColor write FTodayColor default clBlue;
    property RangeColor: TColor read FRangeColor write FRangeColor default clAqua;
    property WeekColor: TColor read FWeekColor write FWeekColor default clWhite;
    property TitleFont: TFont read FTitleFont write SetTitleFont;
    property StartFont: TFont read FStartFont write SetStartFont;
    property FinishFont: TFont read FFinishFont write SetFinishFont;
    property DateFont: TFont read FDateFont write SetDateFont;
    property DateFormat: string read FDateFormat write FDateFormat;
    property Headers: TODCalendarHeaders read FHeaders write FHeaders;
    property BevelEdge: TPanelBevel read FBevelEdge write FBevelEdge default bvRaised;
    property SingleDate: Boolean read FSingleDate write FSingleDate default False;
    property ShowStatus: Boolean read FShowStatus write FShowStatus default True;
    property AutoPage: Boolean read FAutoPage write FAutoPage default True;
    property StartOnMonday: Boolean read FStartOnMonday write FStartOnMonday default False;
    property OnStartChange: TNotifyEvent read FOnStartChange write FOnStartChange;
    property OnFinishChange: TNotifyEvent read FOnFinishChange write FOnFinishChange;
    property OnStartClick: TODDateEvent read FOnStartClick write FOnStartClick;
    property OnFinishClick: TODDateEvent read FOnFinishClick write FOnFinishClick;
    property OnSelectYear: TODSelectYearEvent read FOnSelectYear write FOnSelectYear;
    property OnSetupDay: TODSetupDayEvent read FOnSetupDay write FOnSetupDay;
  end;

  TODPopupCalendar = class(TCustomPanel)
  private
    FForm: TODCalendarForm;
    FText: TEdit;
    FLabel: TLabel;
    FButton: TSpeedButton;
    FHeaders: TODCalendarHeaders;
    FDateFormat, FSeperator, FPopupCaption: string;
    FStartDate, FFinishDate: TDateTime;
    FDisplayYear, FDisplayMonth{, FDisplayWeek}: Integer;
    FDayColor, FTodayColor, FRangeColor, FWeekColor: TColor;
    FTitleFont, FStartFont, FFinishFont, FDateFont: TFont;
    FAutoPage, FSingleDate, FShowStatus, FStartOnMonday: Boolean;
    FBevelEdge: TPanelBevel;
    FPopupHeight, FPopupWidth: Integer;
    FOnStartChange, FOnFinishChange: TNotifyEvent;
    FOnStartClick, FOnFinishClick: TODDateEvent;
    FOnSelectYear: TODSelectYearEvent;
    FOnSetupDay: TODSetupDayEvent;
    FOnChange, FOnPopup: TNotifyEvent;
    function GetDisplayDate: TDateTime;
    procedure SetDisplayDate(Value: TDateTime);
    procedure SetDisplayYear(Value: Integer);
    procedure SetDisplayMonth(Value: Integer);
//  procedure SetDisplayWeek(Value: Integer);
    procedure SetTitleFont(Value: TFont);
    procedure SetStartFont(Value: TFont);
    procedure SetFinishFont(Value: TFont);
    procedure SetDateFont(Value: TFont);
    function GetStartDate: TDateTime;
    function GetFinishDate: TDateTime;
    function GetText: string;
    procedure SetStartDate(Value: TDateTime);
    procedure SetFinishDate(Value: TDateTime);
    procedure SetSingleDate(Value: Boolean);
    procedure SetDateFormat(const Value: string);
    procedure SetSeperator(const Value: string);
  protected
    procedure Resize; override;
    procedure SetText;
    procedure ButtonClick(Sender: TObject);
    procedure TextChange(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
    property StartDate: TDateTime read GetStartDate write SetStartDate;
    property FinishDate: TDateTime read GetFinishDate write SetFinishDate;
    property DisplayDate: TDateTime read GetDisplayDate write SetDisplayDate;
    property Text: string read GetText;
  published
    property DateFormat: string read FDateFormat write SetDateFormat;
    property Seperator: string read FSeperator write SetSeperator;
    property PopupCaption: string read FPopupCaption write FPopupCaption;
    property PopupHeight: Integer read FPopupHeight write FPopupHeight default 250;
    property PopupWidth: Integer read FPopupWidth write FPopupWidth default 240;
    property DisplayYear: Integer read FDisplayYear write SetDisplayYear;
    property DisplayMonth: Integer read FDisplayMonth write SetDisplayMonth;
//  property DisplayWeek: Integer read FDisplayWeek write SetDisplayWeek;
    property DayColor: TColor read FDayColor write FDayColor default clWindow;
    property TodayColor: TColor read FTodayColor write FTodayColor default clBlue;
    property RangeColor: TColor read FRangeColor write FRangeColor default clAqua;
    property WeekColor: TColor read FWeekColor write FWeekColor default clWhite;
    property TitleFont: TFont read FTitleFont write SetTitleFont;
    property StartFont: TFont read FStartFont write SetStartFont;
    property FinishFont: TFont read FFinishFont write SetFinishFont;
    property DateFont: TFont read FDateFont write SetDateFont;
    property Headers: TODCalendarHeaders read FHeaders write FHeaders;
    property BevelEdge: TPanelBevel read FBevelEdge write FBevelEdge default bvRaised;
    property SingleDate: Boolean read FSingleDate write SetSingleDate default False;
    property ShowStatus: Boolean read FShowStatus write FShowStatus default True;
    property StartOnMonday: Boolean read FStartOnMonday write FStartOnMonday default False;
    property AutoPage: Boolean read FAutoPage write FAutoPage default True;
    property Enabled;
    property Color;
    property Ctl3D;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnStartChange: TNotifyEvent read FOnStartChange write FOnStartChange;
    property OnFinishChange: TNotifyEvent read FOnFinishChange write FOnFinishChange;
    property OnStartClick: TODDateEvent read FOnStartClick write FOnStartClick;
    property OnFinishClick: TODDateEvent read FOnFinishClick write FOnFinishClick;
    property OnSelectYear: TODSelectYearEvent read FOnSelectYear write FOnSelectYear;
    property OnSetupDay: TODSetupDayEvent read FOnSetupDay write FOnSetupDay;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnPopup: TNotifyEvent read FOnPopup write FOnPopup;
    property OnEnter;
    property OnExit;
    property OnResize;
 end;

procedure Register;

implementation

{$R *.DFM}

procedure Register;
begin
  RegisterComponents('Orbital', [TODCalendarDialog, TODPopupCalendar]);
//RegisterPropertyEditor(TypeInfo(TODCustomCalendar), TODCalendarDialog, 'Calendar', TClassProperty);
end;

//TODCalendarForm

constructor TODCalendarForm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCalendar := TODCalendar.Create(Self);
  with FCalendar do
  begin
    Parent := Self;
    Align := alClient;
    OnStartChange := StartChange;
    OnFinishChange := FinishChange;
  end;
end;

procedure TODCalendarForm.StartChange(Sender: TObject);
begin
  if Assigned(FOnStartChange) then FOnStartChange(Self);
  OKBtn.Enabled := FCalendar.FinishDate > 0;
end;

procedure TODCalendarForm.FinishChange(Sender: TObject);
begin
  if Assigned(FOnFinishChange) then FOnFinishChange(Self);
  OKBtn.Enabled := FCalendar.FinishDate > 0;
end;

//TODCalendarDialog

constructor TODCalendarDialog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not (csDesigning in ComponentState) then
    FForm := TODCalendarForm.Create(Application);
  FHeaders := TODCalendarHeaders.Create;
  FTitleFont := TFont.Create;
  FStartFont := TFont.Create;
  FFinishFont := TFont.Create;
  FDateFont := TFont.Create;
  FDialogHeight := 250;
  FDialogWidth := 240;
  FCaption := 'Select Date Range';
  FDateFormat := 'dddd, mmmm d, yyyy';
  FDayColor := clWindow;
  FTodayColor := clBlue;
  FRangeColor := clAqua;
  FWeekColor := clWhite;
  FBevelEdge := bvRaised;
  FShowStatus := True;
  FAutoPage := True;
//FFramed := True;
  SetDisplayDate(Date);
end;

destructor TODCalendarDialog.Destroy;
begin
  FHeaders.Free;
  FTitleFont.Free;
  FStartFont.Free;
  FFinishFont.Free;
  FDateFont.Free;
  inherited Destroy;
end;

function TODCalendarDialog.Execute: Boolean;
begin
  with FForm.Calendar do
  begin
    Headers.Assign(FHeaders);
    OnStartClick := FOnStartClick;
    OnFinishClick := FOnFinishClick;
    OnSelectYear := FOnSelectYear;
    OnSetupDay := FOnSetupDay;
    DisplayYear := FDisplayYear;
    DisplayMonth := FDisplayMonth;
//  DisplayWeek := FDisplayWeek;
    DayColor := FDayColor;
    TodayColor := FTodayColor;
    RangeColor := FRangeColor;
    WeekColor := FWeekColor;
    TitleFont := FTitleFont;
    StartFont := FStartFont;
    FinishFont := FFinishFont;
    DateFont := FDateFont;
    BevelEdge := FBevelEdge;
    SingleDate := FSingleDate;
    ShowStatus := FShowStatus;
    StartOnMonday := FStartOnMonday;
    AutoPage := FAutoPage;
    DateFormat := FDateFormat;
    if StartDate > 0 then
      DisplayDate := StartDate else
      DisplayDate := Date;
    StartDate := FStartDate;
    if not FSingleDate then
      Finishdate := FFinishDate;
  end;
  with FForm do
  begin
    OnStartChange := FOnStartChange;
    OnFinishChange := FOnFinishChange;
    Caption := FCaption;
    Height := FDialogHeight;
    Width := FDialogWidth;
    Result := ShowModal = ID_OK;
    if Result then
    begin
      FStartDate := Calendar.StartDate;
      FFinishDate := Calendar.FinishDate;
    end
    else
    begin
      Calendar.StartDate := FStartDate;
      Calendar.FinishDate := FFinishDate;
    end;
  end;
end;

function TODCalendarDialog.GetDisplayDate: TDateTime;
begin
  Result := EncodeDate(FDisplayYear, FDisplayMonth, 1);
end;

procedure TODCalendarDialog.SetDisplayDate(Value: TDateTime);
var
  yr, mo, dy: Word;
begin
  DecodeDate(Value, yr, mo, dy);
  FDisplayYear := yr;
  FDisplayMonth := mo;
end;

procedure TODCalendarDialog.SetDisplayYear(Value: Integer);
begin
  if Value < 1900 then FDisplayYear := 1900
  else if Value > 2100 then FDisplayYear := 2100
  else FDisplayYear := Value;
end;

procedure TODCalendarDialog.SetDisplayMonth(Value: Integer);
begin
  if Value < 1 then FDisplayMonth := 1
  else if Value > 12 then FDisplayMonth := 12
  else FDisplayMonth := Value;
end;
{
procedure TODCalendarDialog.SetDisplayWeek(Value: Integer);
var
  tw: Integer;
begin
  tw := FCalendar.WeeksInYear(FDisplayYear);
  if Value < 1 then Value := 1
  else if Value > tw then Value := tw;
  FDisplayMonth := FForm.Calendar.WeekToMonth(Value);
end;
}
procedure TODCalendarDialog.SetTitleFont(Value: TFont);
begin
  FTitleFont.Assign(Value);
end;

procedure TODCalendarDialog.SetStartFont(Value: TFont);
begin
  FStartFont.Assign(Value);
end;

procedure TODCalendarDialog.SetFinishFont(Value: TFont);
begin
  FFinishFont.Assign(Value);
end;

procedure TODCalendarDialog.SetDateFont(Value: TFont);
begin
  FDateFont.Assign(Value);
end;

//TODPopupCalendar

constructor TODPopupCalendar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 320;
  Height := 21;
  BevelOuter := bvNone;
  BorderStyle := bsSingle;
  Color := clWindow;
  ControlStyle := ControlStyle - [csAcceptsControls];
  FHeaders := TODCalendarHeaders.Create;
  FButton := TSpeedButton.Create(Self);
  with FButton do
  begin
    Parent := Self;
    Font.Style := [fsBold];
    Caption := '...';
    OnClick := ButtonClick;
  end;
  FText := TEdit.Create(Self);
  with FText do
  begin
    Parent := Self;
    AutoSize := False;
    BorderStyle := bsNone;
    Top := 1;
    Left := 1;
    Height := 16;
    OnChange := TextChange;
  end;
  FLabel := TLabel.Create(Self);
  with FLabel do
  begin
    Parent := Self;
    Width := 1;
    Color := clBtnFace;
  end;
  Resize;
  if not (csDesigning in ComponentState) then
    FForm := TODCalendarForm.Create(Self);
  FTitleFont := TFont.Create;
  FStartFont := TFont.Create;
  FFinishFont := TFont.Create;
  FDateFont := TFont.Create;
  FPopupHeight := 250;
  FPopupWidth := 240;
  FPopupCaption := 'Select Date Range';
  FDateFormat := 'dddd, mmmm d, yyyy';
  FSeperator := ' - ';
  FDayColor := clWindow;
  FTodayColor := clBlue;
  FRangeColor := clAqua;
  FWeekColor := clWhite;
  FBevelEdge := bvRaised;
  FShowStatus := True;
  FAutoPage := True;
  SetDisplayDate(Date);
end;

destructor TODPopupCalendar.Destroy;
begin
  FHeaders.Free;
  FTitleFont.Free;
  FStartFont.Free;
  FFinishFont.Free;
  FDateFont.Free;
  inherited Destroy;
end;

procedure TODPopupCalendar.Loaded;
begin
  inherited Loaded;
  Resize;
end;

procedure TODPopupCalendar.ButtonClick(Sender: TObject);
begin
  if Assigned(FOnPopup) then FOnPopup(Self);
  with FForm.Calendar do
  begin
    Headers.Assign(FHeaders);
    OnStartClick := FOnStartClick;
    OnFinishClick := FOnFinishClick;
    OnSelectYear := FOnSelectYear;
    OnSetupDay := FOnSetupDay;
    DisplayYear := FDisplayYear;
    DisplayMonth := FDisplayMonth;
//  DisplayWeek := FDisplayWeek;
    DayColor := FDayColor;
    TodayColor := FTodayColor;
    RangeColor := FRangeColor;
    WeekColor := FWeekColor;
    TitleFont := FTitleFont;
    StartFont := FStartFont;
    FinishFont := FFinishFont;
    DateFont := FDateFont;
    BevelEdge := FBevelEdge;
    SingleDate := FSingleDate;
    ShowStatus := FShowStatus;
    StartOnMonday := FStartOnMonday;
    AutoPage := FAutoPage;
    DateFormat := FDateFormat;
    if StartDate > 0 then
      DisplayDate := StartDate
    else
      DisplayDate := Date;
    StartDate := FStartDate;
    if not FSingleDate then
      FinishDate := FFinishDate;
  end;
  with FForm do
  begin
    OnStartChange := FOnStartChange;
    OnFinishChange := FOnFinishChange;
    Height := FPopupHeight;
    Width := FPopupWidth;
    Caption := FPopupCaption;
    ShowHint := ShowHint;
  end;
  if FForm.ShowModal = ID_OK then
  begin
    FStartDate := FForm.Calendar.StartDate;
    FFinishDate := FForm.Calendar.FinishDate;
    SetText;
  end
  else
  begin
    FForm.Calendar.StartDate := FStartDate;
    FForm.Calendar.FinishDate := FFinishDate;
  end;
end;

procedure TODPopupCalendar.TextChange(Sender: TObject);
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TODPopupCalendar.Resize;
begin
  inherited Resize;
  FButton.Height := Height - 4;
  FButton.Width := FButton.Height;
  FButton.Left := Width - FButton.Width - 4;
  FText.Top := (FButton.Height - FText.Height + 1) div 2;
  FText.Width := Width - FButton.Width - 6;
  FLabel.Height := FButton.Height;
  FLabel.Left := FButton.Left - 1;
end;

function TODPopupCalendar.GetStartDate: TDateTime;
begin
  Result := FStartDate;
end;

function TODPopupCalendar.GetFinishDate: TDateTime;
begin
  Result := FFinishDate;
end;

function TODPopupCalendar.GetText: string;
begin
  Result := FText.Text;
end;

procedure TODPopupCalendar.SetStartDate(Value: TDateTime);
begin
  FStartDate := Value;
  if FStartDate > FFinishDate then
    FFinishDate := FStartDate;
  SetText;
end;

procedure TODPopupCalendar.SetFinishDate(Value: TDateTime);
begin
  FFinishDate := Value;
  if FFinishDate < FStartDate then
    FStartDate := FFinishDate;
  SetText;
end;

procedure TODPopupCalendar.SetDateFormat(const Value: string);
begin
  FDateFormat := Value;
  if not (csLoading in ComponentState) then SetText;
end;

procedure TODPopupCalendar.SetSeperator(const Value: string);
begin
  FSeperator := Value;
  if not (csLoading in ComponentState) then SetText;
end;

procedure TODPopupCalendar.SetText;
begin
  if FStartDate = 0 then Exit;
  if FSingleDate then
    FText.Text := FormatDateTime(FDateFormat, FStartDate)
  else
    FText.Text := FormatDateTime(FDateFormat, FStartDate) +
      FSeperator + FormatDateTime(FDateFormat, FFinishDate);
end;

procedure TODPopupCalendar.SetSingleDate(Value: Boolean);
begin
  FSingleDate := Value;
  SetText;
end;

function TODPopupCalendar.GetDisplayDate: TDateTime;
begin
  Result := EncodeDate(FDisplayYear, FDisplayMonth, 1);
end;

procedure TODPopupCalendar.SetDisplayDate(Value: TDateTime);
var
  yr, mo, dy: Word;
begin
  DecodeDate(Value, yr, mo, dy);
  FDisplayYear := yr;
  FDisplayMonth := mo;
end;

procedure TODPopupCalendar.SetDisplayYear(Value: Integer);
begin
  if Value < 1900 then FDisplayYear := 1900
  else if Value > 2100 then FDisplayYear := 2100
  else FDisplayYear := Value;
end;

procedure TODPopupCalendar.SetDisplayMonth(Value: Integer);
begin
  if Value < 1 then FDisplayMonth := 1
  else if Value > 12 then FDisplayMonth := 12
  else FDisplayMonth := Value;
end;
{
procedure TODPopupCalendar.SetDisplayWeek(Value: Integer);
var
  tw: Integer;
begin
  tw := FCalendar.WeeksInYear(FDisplayYear);
  if Value < 1 then Value := 1
  else if Value > tw then Value := tw;
  FDisplayMonth := FForm.Calendar.WeekToMonth(Value);
end;
}
procedure TODPopupCalendar.SetTitleFont(Value: TFont);
begin
  FTitleFont.Assign(Value);
end;

procedure TODPopupCalendar.SetStartFont(Value: TFont);
begin
  FStartFont.Assign(Value);
end;

procedure TODPopupCalendar.SetFinishFont(Value: TFont);
begin
  FFinishFont.Assign(Value);
end;

procedure TODPopupCalendar.SetDateFont(Value: TFont);
begin
  FDateFont.Assign(Value);
end;

procedure TODCalendarForm.FormShow(Sender: TObject);
begin
end;

end.
